@@ -4,6 +4,7 @@ import pandas as pd
 import numpy as np
 from weakref import WeakSet
 import colorlog
+import re
 
 from ParameterStructure.Instances import InstancePlacementGeometry
 from .config import continue_on_error
@@ -144,6 +145,44 @@ class SimultanObject(object):
     def __del__(self):
         self.__class__._cls_instances_dict_cache = None
 
+    @lru_cache()
+    def get_param_index(self, param):
+        """
+        Return the index of the parameter with the name 'param' in self self._wrapped_obj.ContainedParameters.Items
+        :param str; Example: 'Area'
+
+        """
+        idx = next((i for i, x in enumerate(self._wrapped_obj.ContainedParameters.Items) if x.Name == param), None)
+        return idx
+
+    def get_param(self, param):
+        idx = self.get_param_index(param)
+        if idx is None:
+            raise AttributeError(f'{self} {self.name} {self.id} has no parameter {param}')
+        return self._wrapped_obj.ContainedParameters.Items[idx].get_ValueCurrent()
+
+    def set_param(self, param, value):
+        idx = self.get_param_index(param)
+        if idx is None:
+            raise AttributeError(f'{self} {self.name} has no parameter {param}')
+        self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+
+    @property
+    def current_slot(self):
+        return self._wrapped_obj.CurrentSlot
+
+    @property
+    def fits_in_slot(self):
+        return self._wrapped_obj.FitsInSlots
+
+    @property
+    def slot_extension(self):
+        """
+        Returns the slot extension of the component
+        :param
+        """
+        return int(re.findall("[-+]?[.]?[\d]+(?:,\d\d\d)*[\.]?\d*(?:[eE][-+]?\d+)?", self.current_slot)[-1])
+
 
 class List(UserList):
 
@@ -239,9 +278,10 @@ class BuildInFace(SimultanObject):
 
     @property
     def area(self):
-        idx = next((i for i, x in enumerate(self._wrapped_obj.ContainedParameters.Items) if x.Name == 'A'), None)
-        param = self._wrapped_obj.ContainedParameters.Items[idx]
-        return param.get_ValueCurrent()
+        # idx = next((i for i, x in enumerate(self._wrapped_obj.ContainedParameters.Items) if x.Name == 'A'), None)
+        # param = self._wrapped_obj.ContainedParameters.Items[idx]
+        # return param.get_ValueCurrent()
+        return self.get_param('A')
 
     @property
     def geo_ids(self):
@@ -442,11 +482,343 @@ class BuildInZone(SimultanObject):
         return [x for x in self.contained_components if x._wrapped_obj.FitsInSlots[0] == 'Geometrische_Flächen']
 
 
+class BaseBuildInConstruction(SimultanObject):
+
+    def __init__(self, *args, **kwargs):
+        """
+        Default implementation for a wall construction (InstanceType == InstanceType.ALIGNED_WITH).
+        """
+        SimultanObject.__init__(self, *args, **kwargs)
+        self.check_type()
+
+    @property
+    def is_window(self):
+        if self.get_param_index('gVergl') is None:
+            return False
+        else:
+            return True
+
+    def check_type(self):
+        if self.is_window:
+            object.__setattr__(self, '__class__', BuildInWindowConstruction)
+        else:
+            object.__setattr__(self, '__class__', BuildInWallConstruction)
 
 
+class BuildInWallConstruction(BaseBuildInConstruction):
+
+    def __init__(self, *args, **kwargs):
+        """
+        Default implementation for a wall construction (InstanceType == InstanceType.ALIGNED_WITH).
+        """
+        BaseBuildInConstruction.__init__(self, *args, **kwargs)
+
+    @property
+    def total_thickness(self):
+        idx = self.get_param_index('DickeGes')
+        # idx = next((i for i, x in enumerate(self._wrapped_obj.ContainedParameters.Items) if x.Name == 'DickeGes'), None)
+        param = self._wrapped_obj.ContainedParameters.Items[idx]
+        return param.get_ValueCurrent()
+
+    @property
+    def layers(self):
+        layers = [self._template_parser.create_python_object(x, template_name='BuildInMaterialLayer') for x in self._wrapped_obj.ContainedComponentsAsList]
+
+        # sort the layers by their slot_extension:
+        slot_extensions = [x.slot_extension for x in layers]
+        # test with rotated
+        # layers = layers[1:] + layers[:1]
+
+        return [x for _, x in sorted(zip(slot_extensions, layers))]
 
 
+class BuildInWindowConstruction(BaseBuildInConstruction):
+
+    def __init__(self, *args, **kwargs):
+        """
+        Default implementation for a window construction (InstanceType == InstanceType.ALIGNED_WITH).
+        """
+        BaseBuildInConstruction.__init__(self, *args, **kwargs)
+
+    @property
+    def d(self):
+        """
+        layer thickness in m
+        :param
+        """
+        try:
+            return self.get_param('d')
+        except AttributeError:
+            return self.get_param('Dicke')
+
+    @d.setter
+    def d(self, value):
+        # idx = self.get_param_index('d')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('d', value)
+
+    @property
+    def eps(self):
+        """
+        eps
+        :param
+        """
+        return self.get_param('eps')
+
+    @eps.setter
+    def eps(self, value):
+        # idx = self.get_param_index('eps')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('eps', value)
+
+    @property
+    def gVergl(self):
+        """
+        Gesamtenergiedurchlassgrad
+        :param
+        """
+        return self.get_param('gVergl')
+
+    @gVergl.setter
+    def gVergl(self, value):
+        # idx = self.get_param_index('gVergl')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('gVergl', value)
+
+    @property
+    def curtain_position(self):
+        """
+
+        :param
+        """
+        return self.get_param('LageBehang')
+
+    @curtain_position.setter
+    def curtain_position(self, value):
+        # idx = self.get_param_index('LageBehang')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('LageBehang', value)
+
+    @property
+    def openability(self):
+        """
+        openability of a window; 1=vollständig  0.1=kippen
+        :param
+        """
+        return self.get_param('Öffenbarkeit')
+
+    @openability.setter
+    def openability(self, value):
+        # idx = self.get_param_index('Öffenbarkeit')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('Öffenbarkeit', value)
+
+    @property
+    def frame_width(self):
+        """
+        frame width in m
+        :param
+        """
+        return self.get_param('Rahmenbreite')
+
+    @frame_width.setter
+    def frame_width(self, value):
+        # idx = self.get_param_index('Rahmenbreite')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('Rahmenbreite', value)
+
+    @property
+    def u_f(self):
+        """
+        U-Wert Rahmen in W/m²K
+        :param
+        """
+        return self.get_param('Uf')
+
+    @u_f.setter
+    def u_f(self, value):
+        # idx = self.get_param_index('Uf')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('Uf', value)
+
+    @property
+    def u_g(self):
+        """
+        U-Wert Glas in W/m²K
+        :param
+        """
+        return self.get_param('Ug')
+
+    @u_g.setter
+    def u_g(self, value):
+        self.set_param('Ug', value)
+
+    @property
+    def alpha_curtain(self):
+        """
+        Absorption Behang
+        :param
+        """
+        return self.get_param('αeB')
+
+    @alpha_curtain.setter
+    def alpha_curtain(self, value):
+        self.set_param('αeB', value)
+
+    @property
+    def rho_curtain(self):
+        """
+        Reflexion Behang
+        :param
+        """
+        return self.get_param('ρeB')
+
+    @rho_curtain.setter
+    def rho_curtain(self, value):
+        self.set_param('ρeB', value)
+
+    @property
+    def tau_curtain(self):
+        """
+        Transmission Behang
+        :param
+        """
+        return self.get_param('τeB')
+
+    @tau_curtain.setter
+    def tau_curtain(self, value):
+        self.set_param('τeB', value)
+
+    @property
+    def psi(self):
+        """
+        Transmission Behang
+        :param
+        """
+        return self.get_param('ψ')
+
+    @psi.setter
+    def psi(self, value):
+        self.set_param('ψ', value)
 
 
+class BuildInMaterialLayer(SimultanObject):
 
+    def __init__(self, *args, **kwargs):
+        """
+        Default implementation for a material layer of a wall construction.
+        """
+        SimultanObject.__init__(self, *args, **kwargs)
+        print('done')
 
+    @property
+    def absorption_rate(self):
+        """
+        absorption rate (accoustic?)
+        :param
+        """
+        return self.get_param('aGr')
+
+    @absorption_rate.setter
+    def absorption_rate(self, value):
+        # idx = self.get_param_index('aGr')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('aGr', value)
+
+    @property
+    def c(self):
+        """
+        specific heat capacity in J/kgK
+        :param
+        """
+        return self.get_param('c')
+
+    @c.setter
+    def c(self, value):
+        # idx = self.get_param_index('c')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('c', value)
+
+    @property
+    def d(self):
+        """
+        layer thickness in m
+        :param
+        """
+        return self.get_param('d')
+
+    @d.setter
+    def d(self, value):
+        # idx = self.get_param_index('d')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('d', value)
+
+    @property
+    def w20(self):
+        """
+        Ausgleichsfeuchte bei rli 20%
+        :param
+        """
+        return self.get_param('w20')
+
+    @w20.setter
+    def w20(self, value):
+        # idx = self.get_param_index('w20')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('w20', value)
+
+    @property
+    def w80(self):
+        """
+        Ausgleichsfeuchte bei rli 80%
+        :param
+        """
+        return self.get_param('w80')
+
+    @w80.setter
+    def w80(self, value):
+        # idx = self.get_param_index('w80')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('w80', value)
+
+    @property
+    def heat_conductivity(self):
+        """
+        heat conductivity in W/mK
+        :param
+        """
+        return self.get_param('λ')
+
+    @heat_conductivity.setter
+    def heat_conductivity(self, value):
+        # idx = self.get_param_index('λ')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('λ', value)
+
+    @property
+    def mu(self):
+        """
+        Wasserdampfdiffusionswiderstand
+        :param
+        """
+        return self.get_param('μ')
+
+    @mu.setter
+    def mu(self, value):
+        # idx = self.get_param_index('μ')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('μ', value)
+
+    @property
+    def rho(self):
+        """
+        Rohdichte in kg/m³
+        :param
+        """
+        return self.get_param('ρ')
+
+    @rho.setter
+    def rho(self, value):
+        # idx = self.get_param_index('ρ')
+        # self._wrapped_obj.ContainedParameters.Items[idx].set_ValueCurrent(value)
+        self.set_param('ρ', value)
