@@ -3,6 +3,7 @@ import numpy as np
 import colorlog
 from .utils import classproperty, get_obj_by_id
 
+from .config import cs_axis_up
 from .ParameterStructure.Instances import InstanceType
 # from copy import deepcopy
 # from functools import lru_cache
@@ -284,7 +285,29 @@ class BaseGeometricVertex(BaseGeoBaseClass):
 
     @property
     def position(self):
-        return np.array([self._wrapped_obj.Position.X, self._wrapped_obj.Position.Y, self._wrapped_obj.Position.Z])
+        wrapped_obj = self._wrapped_obj
+        if cs_axis_up == 1:
+            return np.array([wrapped_obj.Position.X, wrapped_obj.Position.Y, wrapped_obj.Position.Z])
+        elif cs_axis_up == 2:
+            return np.array([wrapped_obj.Position.X, wrapped_obj.Position.Z, wrapped_obj.Position.Y])
+        elif cs_axis_up == 3:
+            return np.array([wrapped_obj.Position.Y, wrapped_obj.Position.X, wrapped_obj.Position.Z])
+
+    @position.setter
+    def position(self, value):
+        wrapped_obj = self._wrapped_obj
+        if cs_axis_up == 1:
+            wrapped_obj.Position.X = value[0]
+            wrapped_obj.Position.Y = value[1]
+            wrapped_obj.Position.Z = value[2]
+        elif cs_axis_up == 2:
+            wrapped_obj.Position.X = value[0]
+            wrapped_obj.Position.Y = value[2]
+            wrapped_obj.Position.Z = value[1]
+        elif cs_axis_up == 3:
+            wrapped_obj.Position.X = value[1]
+            wrapped_obj.Position.Y = value[0]
+            wrapped_obj.Position.Z = value[2]
 
 
 class BaseGeometricEdge(BaseGeoBaseClass):
@@ -366,8 +389,12 @@ class BaseGeometricVolume(BaseGeoBaseClass):
         BaseGeoBaseClass.__init__(self, *args, **kwargs)
         self._faces = kwargs.get('faces', None)
 
+    @property
+    def components(self):
+        return self.geometry_model.get_py_geo_components(self._wrapped_obj)
 
-def create_geo_classes():
+
+def create_geo_classes(geo_types):
     """
     Create new classes from geometric base classes
 
@@ -376,24 +403,24 @@ def create_geo_classes():
 
     logger.debug('creating base geo classes')
 
-    class GeoBaseClass(BaseGeoBaseClass):
+    class GeoBaseClass(geo_types['base']):
         pass
 
-    class GeometricLayer(BaseGeometricLayer):
+    class GeometricLayer(geo_types['layer']):
         pass
 
-    class GeometricVertex(BaseGeometricVertex):
+    class GeometricVertex(geo_types['vertex']):
         pass
 
-    class GeometricEdge(BaseGeometricEdge):
+    class GeometricEdge(geo_types['edge']):
         def get_vertices(self):
             return [GeometricVertex.get_obj_by_id(x.Id) for x in self._wrapped_obj.Vertices.Items]
 
-    class GeometricEdgeLoop(BaseGeometricEdgeLoop):
+    class GeometricEdgeLoop(geo_types['edge_loop']):
         def get_edges(self):
             return [GeometricEdge.get_obj_by_id(x.Edge.Id) for x in self._wrapped_obj.Edges.Items]
 
-    class GeometricFace(BaseGeometricFace):
+    class GeometricFace(geo_types['face']):
 
         @property
         def boundary(self):
@@ -403,11 +430,20 @@ def create_geo_classes():
         def holes(self):
             return [GeometricEdgeLoop.get_obj_by_id(x.Id) for x in self._wrapped_obj.Holes.Items]
 
+        @property
+        def hole_faces(self):
+            hole_faces = []
+            for hole in self.holes:
+                hole_face = [x for x in hole._wrapped_obj.Faces if x.Boundary.Id == hole.id][0]
+                hole_faces.append(hole_face)
+            return [GeometricFace.get_obj_by_id(x.Id) for x in hole_faces]
+
         @property
         def points(self):
             return self.boundary.points
 
-    class GeometricVolume(BaseGeometricVolume):
+
+    class GeometricVolume(geo_types['volume']):
 
         def __init__(self, *args, **kwargs):
             BaseGeometricVolume.__init__(self, *args, **kwargs)
